define(['config', 'jquery', 'user_info', 'dashboard/model', 'bus', 'dashboard/widgets/widget', 'app-menu', 'jquery-ui/widget', 'messages', 'utils', 'ajax-indicator', 'dashboard', 'shortcuts', 'theme-ecmwf-2017/bootstrap/index'], function(ECMWF, $, User, Tabs, Bus, DashboardWidget, AppMenu) {

var wrapper, bus = Bus();
var old_template;
var app_menu;

ECMWF.timestamp = new Date()

var Application = {
	ready: $.Deferred(function(df) {
		setTimeout(df.resolve, 10000);
		bus.listen_once('dashboard-set-loaded', df.resolve);
	})
}

$(function() {
	$('#ajax-loader').ajaxIndicator();
	$('#messages').messages();

	if(__HAVE_APPMENU__)
		$('body').addClass('template-2017');
	else
		old_template = true;

	Application.ready.then(function(){
		ECMWF.wheelspeed = ECMWF.browser.mozilla ? 10 : 1;

		$('#main > .loader').fadeOut(function(){
			$('#main').loader('destroy');
		});
		User.info.then(function(user) {
			user.user_name && make_menus();
		})
	});
});

User.info.then(function(user) {

	user.user_name && Tabs.load.then(function(dashboard_set){
		$(function() {
			var main = $('#main > .container-fluid');

			wrapper = main
				.div('#dashboard-set')
					.dashboardSet({dashboard_set: dashboard_set, focused: ECMWF.request.widget, wrapper: '#main'});
					//.selectable({filter: 'li', cancel: '.widget-content', stop: function(e, ui){}});

			User.is_forecaster() && main.add('#header, #footer').css({maxWidth: 10000})

			bus
				.listen('dashboard-set-changed', function(_, set){
					Tabs.save_dashboard_set(set, function(){
						console.log('set saved.');
					});
					update_layout_menu();
				})
				.listen('dashboard-changed', function(_, dashboard){
					dashboard.get_state_async().then(function(get_state) {
						var name = dashboard.options.dashboard.name
						var set = wrapper.dashboardSet('instance')
						if(set.removing[name]) return console.warn('not saving removed dashboard');
						set.saving[name] = true
						Tabs.save_dashboard(get_state(), function(){
							delete set.saving[name];
							console.log('tab saved.');
						});

						update_layout_menu();
					});
				})
				.listen('dashboard-animation-toggle', function(){
					//$.animationAdd();
				});

			$(document)
				.shortcuts({shortcuts: [
					{
						key    : $.ui.keyCode.SPACE,
						signal : 'dashboard-animation-toggle'
					},
					{
						key    : 70, // 'f'
						signal : 'fullscreen'
					}
				]});

			require(['forecaster/reporter'], function(Reporter) { Reporter(); });
		})
	})
})

function update_layout_menu(){
	if(wrapper){
		User.info.then(function() {
			if(app_menu){
				$.each(app_menu.items[1].items, function(i, v) {
					v.is_selected = (+i) === wrapper.dashboardSet('n_columns') - 1;
				})
				app_menu.render()
			}
		})
	}
}

/*
 *  Non-forecasters see a very cut-down 'Add widget' menu, and nothing else.
 */
function make_menus(){
	User.info.then(function(user) {
		var exclude = User.is_forecaster() ? ['basic_chart'] : ['chart', 'meteogram_10_days', 'meteogram_15_days', 'meteogram_efi', 'meteogram_extended']

		var items = [
			{
				name: 'addwidget',
				title: 'Add widget',
				items: $.map(DashboardWidget.registry(), function(widget){
					if(!widget.is_private && !~$.inArray(widget.kind, exclude)){
						return {
							name: widget.kind,
							title: widget.title,
							selected: function(){
								wrapper.dashboardSet('add_widget', {widget: widget, top: true}, function(){
									$.logMessage('New widget added');
								});
							}
						}
					}
				})
			},
			{
				name: 'layout',
				title: 'Layout',
				classes: 'selectable',
				items: $.map(new Array(6), function(v, i){ return {
					name: 'radio-' + (i + 1),
					title: (i + 1) +' Column' + (i ? 's' : ''),
					selected: function(e) {
						wrapper.dashboardSet('reflow', i + 1);
					}
				}}),
				enabled: User.is_forecaster()
			}
		]

		// TODO what should help menu contents be for non-forecasters?
		var help = [{
			name: 'help',
			title: 'Help',
			items: [
				{name: 'help',        title: 'Dashboard help', selected: function(){ window.open('https://software.ecmwf.int/wiki/display/FCST/Chart+dashboard'); }},
				{name: 'preferences', title: 'Preferences', selected: function(){ require(['preferences'], function(){ $(window).preferences() })}},
				{name: 'report',      title: 'Report issue'}
			],
			enabled: User.is_forecaster()
		}]

		var el = $('#app-menu .row')

		app_menu = new AppMenu({
			el: el,
			items: items,
			help: help
		})
		update_layout_menu();
	})
}

return Application
});
