/*
 *
 */
define(['jquery', 'virtual-dom/h', './index.css', 'bootstrap_modal/bootstrap_modal.css'], function($, h) {

var zIndex = 80;

function Dialogue(options){
	var self = this;

	var o = this.options = $.extend({
		title: '',
		id: undefined, // mandatory
		size: undefined,
		position: undefined,
		footer: $.noop,
		resizable: false
	}, options);

	this.size = this.options.size
	this.min_size = {width: 320, height: 320}
	this.position = options.position || {top: 100, left: 100}
	this._open = true;

	o.renderer.on.render.once(function(){
		var $this = $('#' + o.id)
		var header = $('#' + o.id + ' .modal-header')
		var offset
		var down
		var start
		var _pos

		function move_to_top() {
			var group = $.makeArray($('body > .dialogue')).sort(function (a, b) {
				return (parseInt($(a).css('zIndex'), 10) || 0) - (parseInt($(b).css('zIndex'), 10) || 0);
			});

			if(group.length) {
				$(group).each(function(i) {
					this.style.zIndex = zIndex + i;
				});

				$this.css({zIndex: zIndex + group.length});
			}
		}

		header
			.on('mousedown', function(e) {
				if(e.which && e.which !== 1) return;

				down = true
				offset = header.offset();
				start = {x: e.pageX - offset.left, y: e.pageY - offset.top}

				$(document)
					.on('mousemove.dialogue', move)
					.on('mouseup.dialogue', up)

				move_to_top();

				e.preventDefault();
			})
			.on('mouseup', function(e) {
				up(e)
			})
			.on('mousemove', function(e) {
				if(down){
					move(e)
				}
			})

		function move(e) {
			var x = e.pageX - offset.left;
			var y = e.pageY - offset.top;
			var dx = x - start.x;
			var dy = y - start.y;
			$('#' + o.id).css((_pos = {
				left: Math.min(Math.max(self.position.left + dx, 10 - self.size.width), $(window).width() - 20),
				top: Math.min(Math.max(self.position.top + dy, 10 - self.size.height), $(window).height() - 20)
			}))

			e.stopPropagation();
			e.preventDefault();
		}

		function up(e) {
			if(down){
				down = false
				if(_pos){
					self.position = _pos
					_pos = null
				}
				$(document).off('.dialogue')
			}
		}

		var $resize = $this.find('.resize')
		var resize_down, resize_timer;
		var offset2;
		var s;
		$resize
			.on('mousedown', function(e) {
				if(e.which && e.which !== 1) return;

				offset = $resize.offset();
				offset2 = $this.offset();
				s = {x: e.pageX - offset.left, y: e.pageY - offset.top}
				resize_down = true;

				$(document)
					.on('mousemove.dialogue', move2)
					.on('mouseup.dialogue', up2)

				resize_timer = resize_timer || setInterval(function(){ self.options.on_resize(self.size) }, 2000)

				e.preventDefault();
			})
			.on('mouseup', up2)
			.on('mousemove', move2)

		function move2(e) {
			if(resize_down){
				var x = e.pageX - offset2.left;
				var y = e.pageY - offset2.top;
				$this.css((self.size = {width: Math.max(self.min_size.width, x + s.x), height: Math.max(self.min_size.height, y + s.y)}))

				e.preventDefault();
			}
		}

		function up2() {
			if(resize_down){
				resize_down = false
				resize_timer = clearInterval(resize_timer)
				$(document).off('.dialogue')
				self.options.on_resize(self.size)
			}
		}
	})
}

Dialogue.prototype = {

	render: function(body) {
		var self = this, o = this.options;
		var css = {};
		for(var p in this.size) css[p] = this.size[p] + 'px'
		for(var p in this.position) css[p] = this.position[p] + 'px'

		var footer = o.footer()
		if(footer) footer = h('.modal-footer', footer)

		return h('.dialogue#' + o.id + (this._open ? '' : '.closed'), {style: css}, [
			h('.modal-header', [h('h2', [
				o.title,
				h('.close', {onclick: function(){
					console.log('close');
					self.close();
				}})
			])]),
			body,
			footer,
			o.resizable ? h('.resize.ui-icon.ui-icon-grip-diagonal-se') : undefined
		])
	},

	open: function() {
		this._open = true;
		this.options.renderer.render();
	},

	close: function() {
		this._open = false;
		this.options.renderer.render();
	}
}

return Dialogue;

});
