/*
 *  Chart widget using OpenLayers that provides compatibility
 *  with the original ECMWF chart widget, for example for sending
 *  and receiving bus messages. Compatibility may be removed
 *  in the future.
 */
var $ = require('jquery')
import Bus from 'bus'

// http://openlayers.org/en/latest/apidoc/
import Map from 'ol/map'
import View from 'ol/view'
import Tile from 'ol/layer/tile'
import TileWMS from 'ol/source/tilewms'

var Chart = function(args) {
	var self = this;

	var o = this.options = $.extend({
		product: null,
		bus: Bus()
	}, args);

	this.product = o.product

	o.bus
		.listen('layer-delete', function(_, msg) {
			self.remove_layer(msg.layer_id)
		})

	this.map = new Map({
		target: 'map',
		layers: $.map(this.product.state.layers, function(v) {
			return new Tile({
				source: new TileWMS({
					url: 'https://apps.ecmwf.int/wms',
					params: {LAYERS: v.name, token: 'metview'}
				})
			})
		}),
		view: new View({
			projection: 'EPSG:4326',
			center: [0, 0],
			zoom: 3
		})
	})
}

Chart.prototype = {
	remove_layer: function(layer_id) {
		var layer = this.product.layers_by_id[layer_id];
		var index = this.product.layerList().indexOf(layer)

		if(index > -1){
			this._push_undo();
			this.product.layerDelete(layer_id);
			this._clear()._redraw(function() {
				this._notifyProductChange();
			});

			var l = this.map.getLayers().item(index)
			if(l) this.map.removeLayer(l)
		}
	},

	_push_undo: function() {
		return this
	},
	_clear: function() {
		return this
	},
	_redraw: function() {
		return this
	},
	_notifyProductChange: function() {
	},
}

export default Chart;
