/*
 *  Eccharts layer list
 *
 */
import $ from 'jquery'
import RenderTree from 'render-tree'
import virtualise from 'vdom-virtualize'
import h from 'virtual-dom/h'
import Bus from 'bus'
import Layer from 'chart/layer'
import Button from 'vgrid/button'
import Sortable from 'sortablejs'

import 'layer-list/2.css'

function LayerList(options) {
	var self = this;

	var o = this.options = $.extend({
		bus: Bus()
	}, options);

	this.bus = o.bus
	this.items = []

	o.renderer.on.render.once(function() {
		var el = document.querySelector('#layer-list-pane .items');
		var sortable = Sortable.create(el, {
			onSort: function (e) {
				// TODO hide any layer edit forms

				self.bus.send('layer-order-change', {order: sortable.toArray().reverse()});
			}
		})
	})

	this.add_layers = new Button({
		icon: 'ui-icon-circle-plus',
		text: 'Add layers ...',
		show_text: true,
		icon_position: 'left', // TODO
		className: 'add-layer',
		on_click: function() {
			self._open_thumbnail_dialogue();
		}
	})

	o.bus
		.listen('chart-content-update', function(event, msg) {
			self.product = msg.product
			self.time = msg.time

			function add_item(i) {
				var button = new Button({
					renderer: o.renderer,
					icon: 'ui-icon-trash',
					className: 'remove',
					on_click: function(e) {
						var layers = self.product.layerList()
						if(layers.length > 1) {
							button.disable()
							o.bus.send('layer-delete', {layer_id: layers[i].id });
						}
					}
				})

				self.items.push({remove: button})
			}

			// Ensure the items array is the correct length
			// - There is the possibility that self.items gets out of sync
			//   with product.layers - it may be neccesary to rebuild
			//   self.items from scratch here
			var n_layers = self.product.layerList().length;
			if(n_layers < self.items.length){
				// if the delete button is disabled, this indicates that it might be the deleted layer
				var disabled = $.map(self.items, function(l, i){
					if(l.remove.disabled){
						return i;
					}
				})
				$.each(disabled, function(i, l){
					self.items.splice(l, 1)
				})
			}
			self.items = self.items.splice(0, n_layers);
			if(self.items.length < n_layers){
				for(var i=self.items.length;i<n_layers;i++){
					add_item(i)
				}
			}

			self.render();
		})
}

LayerList.prototype = {
	render: function(body) {
		var self = this;

		self.product && $.each(self.product.layerList(), function(i, layer) {
			self.product.layer_icon(layer.id, null, self.time, function(result) {
				var url = result && result.url;
				var changed = url !== self.items[i].thumb
				if(changed){
					self.items[i].thumb = url
					self.options.renderer.render();
				}
			})
		})
		return this._render()
	},

	_render: function(body) {
		var self = this;

		var items = []
		this.product && $.each(self.product.layerList(), function(i, layer) {
			var item = self.items[i]
			items.unshift(h('div', {attributes: {'data-id': layer.id}}, [
				h('img.thumb', {src: self.items[i].thumb}),
				h('span.title', layer.title),
				item.remove.render()
			]))
		})

		return h('.layer-list', [
			this.add_layers.render(),
			h('.items', items)
		])
	},

	_open_thumbnail_dialogue: function() {
		require(['thumbnail-dialogue/2'], function(LayerSelector){
			self.dialogue = self.dialogue
				? self.dialogue.open()
				: new LayerSelector({kind: 'layer', hide_user_items: false})
		})
	}
}

export default LayerList;
