/*
 *  ECMWF Notify
 *
 *  To force notifications to be autoclosing, set a value in minutes for 'notification_autoclose' in the user preferences
 */
define(['jquery', 'markup', 'pnotify/lib/umd/PNotify', 'pnotify/lib/umd/PNotifyButtons', 'pnotify/dist/PNotifyBrightTheme.css'], function($, Markup, PNotify) {

PNotify = PNotify.default;
var stories = {};

var OPTIONS = {
	'service-alert': {type: 'error'},
	'service-allclear': {type: 'success'},
	'error': {type: 'error'},
	'success': {type: 'success'},
	'notice': {type: 'notice'}
}

$.extend(PNotify.defaults, {
	textTrusted: true,
	hide: false,
	modules: {
		Buttons: {
			sticker: false
		}
	}
})

var Notify = function __notify(args, id) {

	if((args === 'close') && id){
		var element = find_by_id(id);
		element && element.pnotify_remove();
		return this;
	}

	if($('body > .ui-pnotify').length > 10) return this;

	var o = $.extend({
		id: null,
		kind: null, // should be one of the OPTIONS above
		title: '',
		message: '',
		created: null,
		story: null,
		delivered: $.noop
	}, args);

	var klass = (o.kind && OPTIONS[o.kind]) || OPTIONS.notice;

	var autoclose;
	if(!klass.hide && ((autoclose = $.getUserPreference('notification_autoclose'), '0') !== '0')){
		klass.hide = true;
		klass.delay = autoclose * 60 * 1000;
	}

	if(o.story){
		var story;
		if((story = stories[o.story])){
			var refs = story.refs;

			function reset_timer(){
				clearTimeout(story.timer);

				story.timer = story.options.hide
					? setTimeout(function(){
							//story.queue_remove(); // FIXME fn doesnt exist
						}, story.delay)
					: null;
			}

			if(o.id){
				var pnotify_options = $.extend({}, klass);
				var opts = {title: o.title, text: o.message}
				if(pnotify_options.type){
					if(story.options.type !== pnotify_options.type){
						$.extend(opts, {
							type: pnotify_options.type,
							hide: pnotify_options.hide
						})
					}
				}
			}

			story.update(opts)
			story.message_id = o.id;
			reset_timer();
			return;
		}
	}

	var instance = PNotify[klass.type]($.extend({}, klass, {
		title: o.title,
		text: Markup.toHTML(o.message),
	}));
	o.story && (stories[o.story] = instance);

	$(instance.refs.container).on('click', '.ui-pnotify-closer', function() {
		o.delivered();
		o.story && delete stories[o.story];
	})

	return this;
};

function find_by_id(id) {
	var element;
	var pnotify = $('body').data('pnotify');
	$.each(pnotify, function(){
		if(this.opts.message_id == id) element = this;
	});
	return element;
}

return Notify;

});
